/*
 * Created on 07-Oct-2004
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package cds.tuto;

import java.util.*;
import java.io.*;

import cds.savot.common.*;
import cds.savot.model.*;
import cds.savot.writer.*;
import cds.savot.pull.*;

/** Cette ajoute une colonne flux (calcule  partir d'une magnitude)  un document VOTable
 * On ne traite que la premire table de la premire RESOURCE
 * @author T .Boch
 */
public class FluxConverter {
	
	public FluxConverter() {}

	
	/** Lit le VOTable dans le flux in, repre la colonne de magnitude, calcule le flux et
	 * ajoute la colonne correspondante, puis rcrit le VOTable produit dans le flux out
	 * 
	 * @param in
	 * @param out
	 * @param magCol nom de la colonne de magnitude
	 * @param zp valeur du zero point
	 */
	public void addFluxColumn(InputStream in, OutputStream out, String magCol, double zp) {
		// Le fichier VOTable est entirement charg en mmoire
		SavotPullParser parser = new SavotPullParser(in, SavotPullEngine.FULL, null);

		// on rcupre l'objet VOTable
		SavotVOTable vot = parser.getVOTable();
		
		SavotResource firstRes = (SavotResource)(vot.getResources().getItemAt(0));
		FieldSet fset = firstRes.getFieldSet(0);
		
		// on parcourt les champs pour trouver l'index du champ de magnitude
		Enumeration e = fset.getItems().elements();
		int idxMag = -1;
		int k=0;
		
		while( e.hasMoreElements() ) {
			SavotField curField = (SavotField)e.nextElement();
			if( curField.getName().equals(magCol) ) {
				idxMag = k;
				break;
			}
			k++;
		}
		
		// si on n'a pas trouv le champ magnitude
		if( idxMag<0 ) {
			System.out.println("Could not find field with name "+magCol);
			return;
		}
		
		// ajout du nouveau champ "flux"  la liste des champs
		SavotField fluxField = new SavotField();
		fluxField.setName("flux");
		fluxField.setUnit("Jy");
		fluxField.setDataType("double");
		fluxField.setDescription("Flux computed from magnitude "+magCol);
		fset.addItem(fluxField);
		
		// on rcupre toutes les lignes de la table
		TRSet tr = firstRes.getTRSet(0);

		
		// pour chaque ligne
		for (int i = 0; i < tr.getItemCount(); i++) {

			// on rcupre l'ensemble des cellules
			TDSet theTDs = tr.getTDSet(i);
			
			// rcupration de la valeur de la magnitude
			double mag = Double.parseDouble(((SavotTD)theTDs.getItemAt(idxMag)).getContent());
			// calcul de la valeur de flux
			double flux = magToFlux(mag, zp);
			
			// cration et ajout du nouveau TD
			SavotTD fluxValTD = new SavotTD();
			fluxValTD.setContent(Double.toString(flux));
			theTDs.addItem(fluxValTD);
		}
		
		// criture de l'objet VOTable (modifi) dans le flux de sortie
		new WriteDocument().generateDocument(vot, out);



	}

	private double magToFlux(double mag, double zp) {
		return Math.pow(10, 0.4*(zp-mag));
	}

	/** Convertit un fichier TSV en un fichier VOTable
	 * 1er arg. : fichier TSV
	 * 2e arg. (facultatif) : nom pour fichier VOTable
	 * si le 2e argument est manquant, on crit le VOTable sur stdout
	 */
	public static void main(String[] args) {
		if( args.length==0 ) {
			System.out.println("Error : please enter the name of the VOTable file");
			return;
		}

		try {
			OutputStream out;
			if( args.length>1 ) out = new FileOutputStream(args[1]);
			else out = System.out;

			InputStream in = new FileInputStream(args[0]);

			new FluxConverter().addFluxColumn(in, out, "Mtot", 8.4577);
		} catch( FileNotFoundException e ) {
			System.out.println("File not found !!");
			e.printStackTrace();
		}

	}
}

