/*
 * Created on 07-Oct-2004
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package cds.tuto;

import java.util.*;
import java.io.*;

import cds.savot.common.*;
import cds.savot.model.*;
import cds.savot.writer.*;

/** Classe permettant la conversion TSV (Tab Separated Value) --> VOTable
 * 
 * @author Thomas Boch [CDS]
 */
public class TSVToVOTable {
	
	public TSVToVOTable() {}

	/** Prend les donnes TSV contenues dans tsvIn, et les ecrit en VOTable
	 *  dans votOut
	 *  Prsupposs :
	 *  - on ignore les lignes vides, commenant par "#" ou par "-----"
	 *  - la premire ligne non ignore contient la description des colonnes
	 *  - la ligne suivante dcrit les units
	 *  
	 *  @param tsvIn flux d'entre au format TSV
	 *  @param votOut flux de sortie
	 */
	public void toVOTable(InputStream tsvIn, OutputStream votOut) {
		DataInputStream dis = new DataInputStream(new BufferedInputStream(tsvIn));
		try {
			// on rcupre les noms des champs et les untis
			String[] fields = getFields(dis);
			String[] units = getUnits(dis);

			// Premire tape : on va crer l'objet VOTable en mmoire, et le "remplir"
			// selon le contenu du flux d'entre
			SavotVOTable vot = new SavotVOTable();

			// ajout d'une RESOURCE au document
			SavotResource res = new SavotResource();
			ResourceSet resSet = new ResourceSet();
			resSet.addItem(res);
			vot.setResources(resSet);

			// ajout d'une TABLE  res
			SavotTable table = new SavotTable();
			TableSet tableSet = new TableSet();
			tableSet.addItem(table);
			res.setTables(tableSet);

			// appel de la mthode qui ajoutera les FIELD  notre table
			addFields(fields, units, table);

			SavotData data = new SavotData();
			table.setData(data);
			SavotTableData tableData = new SavotTableData();
			data.setTableData(tableData);

			// ajout des donnes  notre structure
			addData(dis, tableData);

			// criture de l'objet VOTable en mmoire dans le flux de sortie
			new WriteDocument().generateDocument(vot, votOut);


		} catch( IOException ioe ) {
			ioe.printStackTrace();
		}
		
		
	}

	/** ajoute les donnes de dis  tableData */
	private void addData(DataInputStream dis, SavotTableData tableData) throws IOException {
		TRSet trSet = new TRSet();
		tableData.setTRs(trSet);
		String line;
		while( (line=dis.readLine()) != null ) {
			if( ignoreLine(line) ) continue;
			// dlimiteur : tabulation
			StringTokenizer st = new StringTokenizer(line, "\t");
			String[] cells = new String[st.countTokens()];
			int k=0;
			while( st.hasMoreTokens() ) {
				cells[k++] = st.nextToken();
			}

			SavotTR tr= new SavotTR();
			TDSet tds = new TDSet();
			tr.setTDs(tds);
			trSet.addItem(tr);
			for( int i=0; i<cells.length; i++ ) {
				SavotTD td = new SavotTD();
				// On fixe la valeur de la cellule courante
				td.setContent(cells[i]);
				// On ajoute le td courant au TDSet
				tds.addItem(td);
			}
		}
	}
	
	/** Ajoute  une table l'ensemble des champs dont les noms dont dans fields et les units dans units
	 * @param fields
	 * @param units
	 * @param table
	 */
	private void addFields(String[] fields, String[] units, SavotTable table) {
		FieldSet fset = new FieldSet();
		table.setFields(fset);

		for( int i=0; i<fields.length; i++ ) {
			SavotField field = new SavotField();
			field.setName(fields[i]);

			// Ajout de l'unit du champ
			// attention : units peut avoir une longueur infrieure  fields
			// A COMPLETER : dcommenter la ligne suivante pour grer les units
			//if( i<units.length ) field.setUnit(units[i]);

			fset.addItem(field);
		}
	}

	/** Renvoie la liste des libells des champs
	 * @param dis stream TSV
	 * @return tableau des noms des champs
	 */
	private String[] getFields(DataInputStream dis) throws IOException {
		String line;
		while( (line=dis.readLine()) !=null ) {
			if( ignoreLine(line) ) {
				continue;
			}
			// le dlimiteur est la tabulation
			StringTokenizer st = new StringTokenizer(line, "\t");

			String[] fields = new String[st.countTokens()];
			int i=0;
			while( st.hasMoreTokens() ) {
				fields[i++] = st.nextToken();
			}
			return fields;
		}
		return null;
	}

	/** Renvoie la liste des units
	 * @param dis stream TSV
	 * @return tableau des units
	 */
	private String[] getUnits(DataInputStream dis) throws IOException {
		String line;
		while( (line=dis.readLine()) !=null ) {
			if( ignoreLine(line) ) {
				continue;
			}
			// le dlimiteur est la tabulation
			StringTokenizer st = new StringTokenizer(line, "\t");

			String[] units = new String[st.countTokens()];
			int i=0;
			while( st.hasMoreTokens() ) {
				units[i++] = st.nextToken().replaceAll("\"", "");
			}
			return units;
		}
		return null;
	}

	/** Determine si on doit ignorer la ligne passe en paramtre */
	private boolean ignoreLine(String line) {
		if( line.startsWith("#") || line.startsWith("-----") || line.length()==0 ) {
			return true;
		}
		return false;
	}

	/** Convertit un fichier TSV en un fichier VOTable
	 * 1er arg. : fichier TSV
	 * 2e arg. (facultatif) : nom pour fichier VOTable
	 * si le 2e argument est manquant, on crit le VOTable sur stdout
	 */
	public static void main(String[] args) {
		if( args.length==0 ) {
			System.out.println("Error : please enter the name of the TSV file");
			return;
		}

		try {
			OutputStream out;
			if( args.length>1 ) out = new FileOutputStream(args[1]);
			else out = System.out;

			InputStream in = new FileInputStream(args[0]);

			new TSVToVOTable().toVOTable(in, out);
		} catch( FileNotFoundException e ) {
			System.out.println("File not found !!");
			e.printStackTrace();
		}

	}
}

