#!/usr/bin/perl

# Ce script prend un fichier TSV en entre et le transforme en VOTable
# Le VOTable est crit sur la sortie standard
#
# On fera les prsupposs suivants sur le fichier TSV :
# - la premire ligne non commente et non vide contient la description des champs
# - la ligne suivante contient la description des units
# - une ligne commenant par "-----" est ignore
# T. Boch, 20/01/2006


my $input = $ARGV[0] or die "Entrez le nom du fichier TSV comme argument du script";
my $fieldSet = 0;
my $unitSet = 0; # flags pour indiquer si on a dja trait les fields/les units
my $writeMetadata = 0; # flag pour indiquer si on a dja crit la partie mtadonnes du document VOTable

my @fields; # pour mmoriser les descriptions des champs
my @units; # pour mmoriser les units

open(INPUT,"<$input") or die "Erreur de lecture du fichier $input";

# On parcourt l'ensemble des lignes du fichier TSV
while(<INPUT>) {
	chomp;
	
	# On ignore les lignes vides et celles commenant par '#' ou par "-----"
	if ( ($_ =~ m/^\#/) || ($_ =~ m/^$/) || ($_ =~ m/-----/ ) ) {
		next;
	}

	# Traitement de la ligne dcrivant les champs
	if( !$fieldSet ) {
		@fields = split(/\t/,$_);
		$fieldSet = 1;
		next;
	}

	# Traitement de la ligne dcrivant les units
	if( !$unitSet ) {
		@units = split(/\t/,$_);
		$unitSet = 1;
		next;
	}

	if( !$writeMetadata ) {
		&writeMetadata;
		$writeMetadata = 1;
		next;
	}

	
	# On rcupre le tableau de donnes pour la ligne courante
	my @data = split(/\t/,$_);
	print "        <TR>";
	# boucle sur chaque lment de la ligne
	foreach $curData (@data) {
		print "<TD>$curData</TD>";
	}
	print "</TR>\n";

}
close(INPUT);

# Ecriture de la partie finale du document
# A COMPLETER : une balise n'a pas t referme ...
print <<EOM;
        </TABLEDATA>
      </DATA>
    </TABLE>
  </VOTABLE>
EOM

# crit le dbut du document VOTable + la partie metadata
sub writeMetadata {
	# prambule
	print <<EOM;
<?xml version="1.0"?>
<VOTABLE version="1.1" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
 xsi:noNamespaceSchemaLocation="xmlns=http://www.ivoa.net/xml/VOTable/v1.1">
  <RESOURCE>
    <TABLE>
EOM
	# Ecriture des diffrents champs
	$nbFields = scalar(@fields);
	for( $i=0; $i<$nbFields; $i++ ) {
		$curUnit = $units[$i];
		# On supprime les quotes '"'
		$curUnit =~ s/"//g;
		print "      <FIELD name=\"$fields[$i]\" unit=\"$curUnit\" datatype=\"char\" arraysize=\"*\" />\n";
	}

	print <<EOM;
      <DATA>
        <TABLEDATA>
EOM
}

1;
